package com.bureau.devicefingerprint.tools

import com.bureau.devicefingerprint.security.SecureJNI
import com.google.gson.GsonBuilder
import java.nio.charset.StandardCharsets
import java.security.MessageDigest
import javax.crypto.Cipher
import javax.crypto.spec.IvParameterSpec
import javax.crypto.spec.SecretKeySpec
import kotlin.text.Charsets.UTF_8

object DeviceDataEncryptionUtils {
    private val md5HashValueKey = SecureJNI.getPKP1c()
    private val dummyKey = SecureJNI.getPKP2c()
    private val dummyValue = SecureJNI.getPKP3c()
    private val ivGlobal = SecureJNI.getPKP4c().toByteArray(StandardCharsets.UTF_8)
    private val passPhrase = SecureJNI.getPKP5c().toByteArray(StandardCharsets.UTF_8)

    private const val TAG = "DeviceDataEncryption"

    private fun ByteArray.toHex() = joinToString(separator = "") { byte -> "%02x".format(byte) }

    fun <T> getEncryptedPayload(payLoad: T): String {
        val md5EncryptedPayLoad = getMD5EncryptedPayLoad(payLoad) ?: ""
        return getAESEncryptedPayLoad(md5EncryptedPayLoad)
    }

    private fun <T> getMD5EncryptedPayLoad(payLoad: T): String? {
        val gson = GsonBuilder()
            .serializeNulls()
            .disableHtmlEscaping()
            .create()
        val payloadStringifiedBeforeSortingKeys: String = gson.toJson(payLoad)
        val payLoadTreeMap = JSONUtils.jsonToMap(payloadStringifiedBeforeSortingKeys)
        payLoadTreeMap.put(dummyKey, dummyValue)
        val payloadStringifiedAfterSortingKeys = gson.toJson(payLoadTreeMap)
        val crypt = MessageDigest.getInstance("MD5")
            .digest(payloadStringifiedAfterSortingKeys.toByteArray(UTF_8))
        payLoadTreeMap.put(md5HashValueKey, crypt.toHex())
        payLoadTreeMap.remove(dummyKey)
//        DebugUtils.d(TAG, "main: $payLoad")
//        DebugUtils.d(TAG, "main: $payloadStringifiedBeforeSortingKeys")
//        DebugUtils.d(TAG, "main: $payloadStringifiedAfterSortingKeys")

        val finalMd5EncryptedPayLoad = gson.toJson(payLoadTreeMap)
        DebugUtils.d(TAG, "main: $finalMd5EncryptedPayLoad")
        return finalMd5EncryptedPayLoad
    }

    private fun encrypt(key: SecretKeySpec, data: ByteArray): ByteArray {
        val cipher = Cipher.getInstance("AES/CBC/PKCS5Padding")
        cipher.init(Cipher.ENCRYPT_MODE, key, IvParameterSpec(ivGlobal))
        return cipher.doFinal(data)
    }

    private fun arrayBufferToHexString(buffer: ByteArray): String {
        return buffer.joinToString("") { byte -> "%02x".format(byte) }
    }

    private val HEX_ARRAY = "0123456789ABCDEF".toByteArray(StandardCharsets.US_ASCII)
    private fun bytesToHex(bytes: ByteArray): String {
        val hexChars = ByteArray(bytes.size * 2)
        for (j in bytes.indices) {
            val v = bytes[j].toInt() and 0xFF
            hexChars[j * 2] = HEX_ARRAY[v ushr 4]
            hexChars[j * 2 + 1] = HEX_ARRAY[v and 0x0F]
        }
        return String(hexChars, StandardCharsets.UTF_8)
    }

    private fun getAESEncryptedPayLoad(payLoad: String): String {
        val key = SecretKeySpec(passPhrase, "AES")
        val data = payLoad.toByteArray(StandardCharsets.UTF_8)
        val encrypted = encrypt(key, data)
        //DebugUtils.d(TAG, "main1: $encryptedString")
        return bytesToHex(encrypted)
    }
}
